import useGPodder from '@/api/gpodder.js'
import { useLogger } from '@/logger.js'

const logger = useLogger();

function errorHandler(error) {
    if (error.response) {
        // The request was made and the server responded with a status code
        // that falls out of the range of 2xx
        console.log(error.response.data);
        console.log(error.response.status);
        console.log(error.response.headers);

        switch (error.response.status) {
            case 400: logger.badRequestError(); break;
            case 401: logger.unauthorizedError(); break;
            case 404: logger.notFoundError(); break;
        }

    } else if (error.request) {
        // The request was made but no response was received
        // `error.request` is an instance of XMLHttpRequest in the browser and an instance of
        // http.ClientRequest in node.js
        logger.connectionLostError()

        console.log(error.request);
    } else {
        // Something happened in setting up the request that triggered an Error
        console.log('Error', error.message);
        logger.append({
            type: "danger",
            message: err.message
        });
    }
}

const backendURL = import.meta.env.VITE_BACKEND_URL || "http://localhost:8080";
console.log("Backend-URL", backendURL);

const pseSquared = useGPodder({
    // baseURL: process.env.VUE_APP_BASE_URL || "http://localhost:8080",
    // baseURL: "http://api.pse-squared.de",
    baseURL: backendURL,
    throwHandler: error => errorHandler(error)
});

export const {
    changePassword,
    deleteAccount,
    forgotPassword,
    getEpisodeActions,
    getTitles,
    login,
    logout,
    postEpisodeActions,
    postSubscriptions,
    putSubscriptions,
    register,
    resetPassword,
} = pseSquared;

