package org.psesquared.server.authentication.api.controller;

/**
 * This record wraps a dummy device that is required to be returned by <br>
 * {@link AuthenticationController#getDeviceList(String,
 * jakarta.servlet.http.HttpServletResponse)}.
 *
 * @param id            The device id
 * @param caption       The caption, i.e. name, of the device
 * @param type          The device type
 * @param subscriptions The number of subscriptions of the device
 */
public record DeviceWrapper(
    String id,
    String caption,
    String type,
    int subscriptions) {

  /**
   * The id of the dummy device.
   */
  private static final String DUMMY_ID = "dummy";

  /**
   * The name of the dummy device.
   */
  private static final String DUMMY_DEVICE = "device";

  /**
   * The type of the dummy device.
   */
  private static final String DUMMY_TYPE = "other";

  /**
   * The number of subscriptions of the dummy device.
   */
  private static final int DUMMY_SUBSCRIPTIONS = 0;

  /**
   * The no-args-constructor for a device-wrapper containing a dummy device.
   */
  public DeviceWrapper() {
    this(DUMMY_ID, DUMMY_DEVICE, DUMMY_TYPE, DUMMY_SUBSCRIPTIONS);
  }

}
