package org.psesquared.server.episode.actions.api.controller;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import lombok.Data;

/**
 * The Response Object for a GET-Request concerning an EpisodeAction.
 * <br>
 * May contain multiple EpisodeActions.
 */
@Data
public class EpisodeActionGetResponse {

  /**
   * The list of EpisodeActionPosts.
   */
  private final List<EpisodeActionPost> actions;

  /**
   * The timestamp of the response.
   */
  private final long timestamp;

  /**
   * Instantiates a new EpisodeActionGetResponse with the current timestamp.
   *
   * @param episodeActionPosts A list of EpisodeActionPosts
   */
  public EpisodeActionGetResponse(
      final List<EpisodeActionPost> episodeActionPosts) {
    this.actions = episodeActionPosts;
    this.timestamp = LocalDateTime.now().toEpochSecond(ZoneOffset.UTC);
  }

}
