package org.psesquared.server.episode.actions.api.controller;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonUnwrapped;
import jakarta.validation.constraints.NotBlank;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.psesquared.server.model.EpisodeAction;

/**
 * An Episode Action that is being sent to the server via a POST Request.
 * <br>
 * If the user listened to an episode or did another action, an
 * EpisodeActionPOST is uploaded.
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class EpisodeActionPost {

  /**
   * The URL of the podcast the posted episode action belongs to.
   */
  @JsonProperty(value = "podcast", required = true)
  @NotBlank
  private String podcastUrl;

  /**
   * The URL of the corresponding episode.
   */
  @JsonProperty(value = "episode", required = true)
  @NotBlank
  private String episodeUrl;

  /**
   * The title of the corresponding episode.
   */
  private String title;

  /**
   * The GUID of the corresponding episode.
   */
  private String guid;

  /**
   * The total length of the corresponding episode in milliseconds.
   */
  private int total;

  /**
   * The actual episode action whose attributes are presented unwrapped.
   *
   * @see JsonUnwrapped
   */
  @JsonUnwrapped
  private EpisodeAction episodeAction;

}
