package org.psesquared.server.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.NamedAttributeNode;
import jakarta.persistence.NamedEntityGraph;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.NonNull;

/**
 * A podcast that was subscribed.
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "subscriptions")
@NamedEntityGraph(name = "graph.Subscription.episodes",
    attributeNodes = @NamedAttributeNode("episodes"))
public class Subscription implements Serializable {

  /**
   * A primary key for the table.
   */
  @JsonIgnore
  @Id
  @GeneratedValue(strategy=GenerationType.SEQUENCE)
  @Column(name = "id", updatable = false)
  private Long id;

  /**
   * The URL for the RSS-Feed of the Podcast.
   */
  @Column(name = "url", nullable = false)
  private String url;

  /**
   * The title of the Podcast.
   */
  @Column(name = "title")
  private String title;

  /**
   * Timestamp of the last time the RSS-Feed was fetched.
   */
  @Column(name = "timestamp")
  private long timestamp;

  /**
   * The list of SubscriptionActions of this podcast.
   */
  @JsonIgnore
  @OneToMany(mappedBy = "subscription",
      cascade = CascadeType.REMOVE)
  private List<SubscriptionAction> subscriptionActions;

  /**
   * The episodes of a subscription.
   */
  @JsonIgnore
  @OneToMany(mappedBy = "subscription", cascade = CascadeType.REMOVE)
  private final List<Episode> episodes = new ArrayList<>();

  /**
   * Adds an episode to the list of episodes.
   *
   * @param episode The to be added episode
   */
  public void addEpisode(@NonNull final Episode episode) {
    this.episodes.add(episode);
  }

}
